@@ -5,6 +5,15 @@ from .checks.base import GeneralCheck
 from .checks.base import NotNullCheck
 from .checks.base import QueryCheck
 from .checks.base import RangeCheck
+from .checks.cross_section_definitions import CrossSectionEqualElementsCheck
+from .checks.cross_section_definitions import (
+    CrossSectionFirstElementNonZeroCheck,
+)
+from .checks.cross_section_definitions import CrossSectionFloatCheck
+from .checks.cross_section_definitions import CrossSectionFloatListCheck
+from .checks.cross_section_definitions import CrossSectionIncreasingCheck
+from .checks.cross_section_definitions import CrossSectionNonZeroCheck
+from .checks.cross_section_definitions import CrossSectionNullCheck
 from .checks.factories import generate_enum_checks
 from .checks.factories import generate_foreign_key_checks
 from .checks.factories import generate_geometry_checks
@@ -17,7 +26,6 @@ from .checks.other import BoundaryCondition1DObjectNumberCheck
 from .checks.other import ConnectionNodesDistance
 from .checks.other import ConnectionNodesLength
 from .checks.other import CrossSectionLocationCheck
-from .checks.other import CrossSectionShapeCheck
 from .checks.other import OpenChannelsWithNestedNewton
 from .checks.other import TimeseriesCheck
 from .checks.other import Use0DFlowCheck
@@ -192,7 +200,6 @@ CHECKS += [
 ## 005x: CROSS SECTIONS
 
 CHECKS += [
-    CrossSectionShapeCheck(error_code=51),
     CrossSectionLocationCheck(max_distance=1.0, error_code=52),
     OpenChannelsWithNestedNewton(error_code=53),
     QueryCheck(
@@ -203,7 +210,7 @@ CHECKS += [
             models.CrossSectionLocation.reference_level
             > models.CrossSectionLocation.bank_level,
         ),
-        message="CrossSectionLocation.reference_level should be below the CrossSectionLocation.bank_level"
+        message="CrossSectionLocation.reference_level should be below the CrossSectionLocation.bank_level "
         "when CrossSectionLocation.bank_level is not null",
     ),
     QueryCheck(
@@ -274,6 +281,94 @@ CHECKS += [
     BoundaryCondition1DObjectNumberCheck(error_code=72),
 ]
 
+## 008x: CROSS SECTION DEFINITIONS
+
+CHECKS += [
+    CrossSectionNullCheck(
+        error_code=81,
+        column=models.CrossSectionDefinition.width,
+        shapes=None,  # all shapes
+    ),
+    CrossSectionNullCheck(
+        error_code=82,
+        column=models.CrossSectionDefinition.height,
+        shapes=(
+            constants.CrossSectionShape.CLOSED_RECTANGLE,
+            constants.CrossSectionShape.TABULATED_RECTANGLE,
+            constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+        ),
+    ),
+    CrossSectionFloatCheck(
+        error_code=83,
+        column=models.CrossSectionDefinition.width,
+        shapes=(
+            constants.CrossSectionShape.RECTANGLE,
+            constants.CrossSectionShape.CIRCLE,
+            constants.CrossSectionShape.CLOSED_RECTANGLE,
+            constants.CrossSectionShape.EGG,
+        ),
+    ),
+    CrossSectionFloatCheck(
+        error_code=84,
+        column=models.CrossSectionDefinition.height,
+        shapes=(constants.CrossSectionShape.CLOSED_RECTANGLE,),
+    ),
+    CrossSectionNonZeroCheck(
+        error_code=85,
+        level=CheckLevel.WARNING,
+        column=models.CrossSectionDefinition.width,
+        shapes=(
+            constants.CrossSectionShape.RECTANGLE,
+            constants.CrossSectionShape.CIRCLE,
+            constants.CrossSectionShape.CLOSED_RECTANGLE,
+            constants.CrossSectionShape.EGG,
+        ),
+    ),
+    CrossSectionNonZeroCheck(
+        error_code=86,
+        level=CheckLevel.WARNING,
+        column=models.CrossSectionDefinition.height,
+        shapes=(constants.CrossSectionShape.CLOSED_RECTANGLE,),
+    ),
+    CrossSectionFloatListCheck(
+        error_code=87,
+        column=models.CrossSectionDefinition.width,
+        shapes=(
+            constants.CrossSectionShape.TABULATED_RECTANGLE,
+            constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+        ),
+    ),
+    CrossSectionFloatListCheck(
+        error_code=88,
+        column=models.CrossSectionDefinition.height,
+        shapes=(
+            constants.CrossSectionShape.TABULATED_RECTANGLE,
+            constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+        ),
+    ),
+    CrossSectionEqualElementsCheck(
+        error_code=89,
+        shapes=(
+            constants.CrossSectionShape.TABULATED_RECTANGLE,
+            constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+        ),
+    ),
+    CrossSectionIncreasingCheck(
+        error_code=90,
+        column=models.CrossSectionDefinition.height,
+        shapes=(
+            constants.CrossSectionShape.TABULATED_RECTANGLE,
+            constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+        ),
+    ),
+    CrossSectionFirstElementNonZeroCheck(
+        error_code=91,
+        level=CheckLevel.WARNING,
+        column=models.CrossSectionDefinition.width,
+        shapes=(constants.CrossSectionShape.TABULATED_RECTANGLE,),
+    ),
+]
+
 
 ## 01xx: LEVEL CHECKS
 
@@ -503,15 +598,6 @@ CHECKS += [
 ## 030x: SETTINGS
 
 CHECKS += [
-    QueryCheck(
-        error_code=301,
-        level=CheckLevel.WARNING,
-        column=models.GlobalSetting.frict_type,
-        invalid=Query(models.GlobalSetting).filter(
-            models.GlobalSetting.frict_type == None
-        ),
-        message="The global friction type (v2_global_settings.friction_type) is not defined. In the future, this will lead to an error.",
-    ),
     QueryCheck(
         error_code=302,
         column=models.GlobalSetting.dem_obstacle_height,
@@ -551,15 +637,73 @@ CHECKS += [
     RangeCheck(
         error_code=306,
         column=models.GlobalSetting.dist_calc_points,
-        filters=or_(
-            *[
-                Query(table).filter(table.dist_calc_points == None).exists()
-                for table in [models.Channel, models.Pipe, models.Culvert]
-            ]
-        ),
         min_value=0,
         left_inclusive=False,  # 0 itself is not allowed
     ),
+    RangeCheck(
+        error_code=307,
+        column=models.GlobalSetting.grid_space,
+        min_value=0,
+        left_inclusive=False,  # 0 itself is not allowed
+    ),
+    RangeCheck(
+        error_code=308,
+        column=models.GlobalSetting.embedded_cutoff_threshold,
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=309,
+        column=models.GlobalSetting.max_angle_1d_advection,
+        min_value=0,
+        max_value=0.5 * 3.14159,
+    ),
+    RangeCheck(
+        error_code=310,
+        column=models.GlobalSetting.table_step_size,
+        min_value=0,
+        left_inclusive=False,
+    ),
+    RangeCheck(
+        error_code=311,
+        column=models.GlobalSetting.table_step_size_1d,
+        min_value=0,
+        left_inclusive=False,
+    ),
+    RangeCheck(
+        error_code=312,
+        column=models.GlobalSetting.table_step_size_volume_2d,
+        min_value=0,
+        left_inclusive=False,
+    ),
+    RangeCheck(
+        error_code=313,
+        column=models.GlobalSetting.frict_coef,
+        filters=models.GlobalSetting.frict_type == constants.FrictionType.MANNING,
+        min_value=0,
+        max_value=1,
+    ),
+    RangeCheck(
+        error_code=314,
+        column=models.GlobalSetting.frict_coef,
+        filters=models.GlobalSetting.frict_type == constants.FrictionType.CHEZY,
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=315,
+        column=models.GlobalSetting.interception_global,
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=316,
+        column=models.GlobalSetting.manhole_storage_area,
+        min_value=0,
+    ),
+    QueryCheck(
+        error_code=317,
+        column=models.GlobalSetting.epsg_code,
+        invalid=CONDITIONS["has_no_dem"].filter(models.GlobalSetting.epsg_code == None),
+        message="v2_global_settings.epsg_code may not be NULL if no dem file is provided",
+    ),
 ]
 
 ## 04xx: Groundwater, Interflow & Infiltration
@@ -598,12 +742,13 @@ CHECKS += [
         column=models.GroundWater.equilibrium_infiltration_rate,
         invalid=Query(models.GroundWater).filter(
             models.GroundWater.global_settings != None,
-            models.GroundWater.equilibrium_infiltration_rate == None,
+            (models.GroundWater.equilibrium_infiltration_rate == None)
+            | (models.GroundWater.equilibrium_infiltration_rate < 0),
             is_none_or_empty(
                 models.GroundWater.equilibrium_infiltration_rate_file,
             ),
         ),
-        message="a global equilibrium infiltration rate (v2_groundwater.equilibrium_infiltration_rate) should be defined when not using an equilibrium infiltration rate file.",
+        message="a global equilibrium infiltration rate (v2_groundwater.equilibrium_infiltration_rate) should be defined and >=0 when not using an equilibrium infiltration rate file.",
     ),
     QueryCheck(
         error_code=406,
@@ -620,10 +765,11 @@ CHECKS += [
         column=models.GroundWater.infiltration_decay_period,
         invalid=Query(models.GroundWater).filter(
             models.GroundWater.global_settings != None,
-            models.GroundWater.infiltration_decay_period == None,
+            (models.GroundWater.infiltration_decay_period == None)
+            | (models.GroundWater.infiltration_decay_period <= 0),
             is_none_or_empty(models.GroundWater.infiltration_decay_period_file),
         ),
-        message="a global infiltration decay period (v2_groundwater.infiltration_decay_period) should be defined when not using an infiltration decay period file.",
+        message="a global infiltration decay period (v2_groundwater.infiltration_decay_period) should be defined and >0 when not using an infiltration decay period file.",
     ),
     QueryCheck(
         error_code=408,
@@ -674,10 +820,11 @@ CHECKS += [
         column=models.GroundWater.initial_infiltration_rate,
         invalid=Query(models.GroundWater).filter(
             models.GroundWater.global_settings != None,
-            models.GroundWater.initial_infiltration_rate == None,
+            (models.GroundWater.initial_infiltration_rate == None)
+            | (models.GroundWater.initial_infiltration_rate < 0),
             is_none_or_empty(models.GroundWater.initial_infiltration_rate_file),
         ),
-        message="a global initial infiltration rate (v2_groundwater.initial_infiltration_rate) should be defined when not using an initial infiltration rate file.",
+        message="a global initial infiltration rate (v2_groundwater.initial_infiltration_rate) should be defined and >=0 when not using an initial infiltration rate file.",
     ),
     QueryCheck(
         error_code=413,
@@ -725,7 +872,8 @@ CHECKS += [
         column=models.Interflow.porosity_layer_thickness,
         invalid=Query(models.Interflow).filter(
             models.Interflow.global_settings != None,
-            models.Interflow.porosity_layer_thickness > 0,
+            (models.Interflow.porosity_layer_thickness == None)
+            | (models.Interflow.porosity_layer_thickness <= 0),
             models.Interflow.interflow_type
             in [
                 constants.InterflowType.LOCAL_DEEPEST_POINT_SCALED_POROSITY,
@@ -753,11 +901,12 @@ CHECKS += [
         column=models.Interflow.hydraulic_conductivity,
         invalid=Query(models.Interflow).filter(
             models.Interflow.global_settings != None,
-            models.Interflow.hydraulic_conductivity == None,
+            (models.Interflow.hydraulic_conductivity == None)
+            | (models.Interflow.hydraulic_conductivity < 0),
             is_none_or_empty(models.Interflow.hydraulic_conductivity_file),
             models.Interflow.interflow_type != constants.InterflowType.NO_INTERLFOW,
         ),
-        message="v2_interflow.hydraulic_conductivity cannot be null when no hydraulic conductivity file is supplied.",
+        message="v2_interflow.hydraulic_conductivity cannot be null or negative when no hydraulic conductivity file is supplied.",
     ),
     RangeCheck(
         error_code=420,
@@ -766,6 +915,16 @@ CHECKS += [
         min_value=0,
         max_value=1,
     ),
+    QueryCheck(
+        error_code=421,
+        column=models.GroundWater.groundwater_hydro_connectivity,
+        invalid=Query(models.GroundWater).filter(
+            models.GroundWater.global_settings != None,
+            (models.GroundWater.groundwater_hydro_connectivity < 0),
+            is_none_or_empty(models.GroundWater.groundwater_hydro_connectivity_file),
+        ),
+        message="the global hydro connectivity (v2_groundwater.groundwater_hydro_connectivity) should be >=0 when not using an hydro connectivity file.",
+    ),
 ]
 
 
@@ -916,26 +1075,53 @@ CHECKS += [
         column=models.GroundWater.groundwater_hydro_connectivity_file,
         filters=models.GroundWater.global_settings != None,
     ),
+    FileExistsCheck(
+        error_code=714,
+        column=models.GroundWater.leakage_file,
+        filters=(models.GroundWater.global_settings != None),
+    ),
+    FileExistsCheck(
+        error_code=715,
+        column=models.GlobalSetting.initial_waterlevel_file,
+    ),
+    FileExistsCheck(
+        error_code=716,
+        column=models.GlobalSetting.initial_groundwater_level_file,
+        filters=(models.GlobalSetting.groundwater_settings_id != None),
+    ),
 ]
 
 
-## 11xx: SIMULATION SETTINGS
-
+## 110x: SIMULATION SETTINGS, timestep
 CHECKS += [
-    GeneralCheck(
+    QueryCheck(
         error_code=1101,
         column=models.GlobalSetting.maximum_sim_time_step,
-        criterion_valid=models.GlobalSetting.maximum_sim_time_step
-        >= models.GlobalSetting.sim_time_step,
+        invalid=Query(models.GlobalSetting).filter(
+            models.GlobalSetting.maximum_sim_time_step
+            < models.GlobalSetting.sim_time_step
+        ),
+        message="v2_global_settings.maximum_sim_time_step must be greater than or equal to v2_global_settings.sim_time_step",
     ),
-    GeneralCheck(
+    QueryCheck(
         error_code=1102,
         column=models.GlobalSetting.sim_time_step,
-        criterion_valid=models.GlobalSetting.sim_time_step
-        >= models.GlobalSetting.minimum_sim_time_step,
+        invalid=Query(models.GlobalSetting).filter(
+            models.GlobalSetting.minimum_sim_time_step
+            > models.GlobalSetting.sim_time_step
+        ),
+        message="v2_global_settings.minimum_sim_time_step must be less than or equal to v2_global_settings.sim_time_step",
     ),
     QueryCheck(
         error_code=1103,
+        column=models.GlobalSetting.output_time_step,
+        invalid=Query(models.GlobalSetting).filter(
+            models.GlobalSetting.output_time_step < models.GlobalSetting.sim_time_step
+        ),
+        message="v2_global_settings.output_time_step must be greater than or equal to v2_global_settings.sim_time_step",
+    ),
+    QueryCheck(
+        error_code=1104,
         column=models.GlobalSetting.maximum_sim_time_step,
         invalid=Query(models.GlobalSetting).filter(
             models.GlobalSetting.timestep_plus == True,
@@ -944,37 +1130,186 @@ CHECKS += [
         message="v2_global_settings.maximum_sim_time_step cannot be null when "
         "v2_global_settings.timestep_plus is True",
     ),
-    RangeCheck(
-        error_code=1104,
-        column=models.NumericalSettings.convergence_eps,
-        filters=models.NumericalSettings.global_settings != None,
-        min_value=0,
-        left_inclusive=False,
-    ),
-    RangeCheck(
-        error_code=1105,
-        column=models.NumericalSettings.convergence_cg,
-        filters=models.NumericalSettings.global_settings != None,
-        min_value=0,
-        left_inclusive=False,
-    ),
-    RangeCheck(
-        error_code=1106,
-        column=models.NumericalSettings.general_numerical_threshold,
-        filters=models.NumericalSettings.global_settings != None,
-        min_value=0,
-        left_inclusive=False,
-    ),
-    RangeCheck(
-        error_code=1107,
-        column=models.NumericalSettings.flow_direction_threshold,
-        filters=models.NumericalSettings.global_settings != None,
-        min_value=0,
-        left_inclusive=False,
-    ),
 ]
 CHECKS += [
-    TimeseriesCheck(col, error_code=1108)
+    RangeCheck(
+        error_code=1105,
+        column=getattr(models.GlobalSetting, name),
+        min_value=0,
+        left_inclusive=False,
+    )
+    for name in (
+        "sim_time_step",
+        "minimum_sim_time_step",
+        "maximum_sim_time_step",
+        "output_time_step",
+    )
+]
+
+## 111x - 114x: SIMULATION SETTINGS, numerical
+
+CHECKS += [
+    RangeCheck(
+        error_code=1110,
+        column=models.NumericalSettings.cfl_strictness_factor_1d,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=0,
+        left_inclusive=False,
+    ),
+    RangeCheck(
+        error_code=1111,
+        column=models.NumericalSettings.cfl_strictness_factor_2d,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=0,
+        left_inclusive=False,
+    ),
+    RangeCheck(
+        error_code=1112,
+        column=models.NumericalSettings.convergence_eps,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=1e-7,
+        max_value=1e-4,
+    ),
+    RangeCheck(
+        error_code=1113,
+        column=models.NumericalSettings.convergence_cg,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=1e-12,
+        max_value=1e-7,
+    ),
+    RangeCheck(
+        error_code=1114,
+        column=models.NumericalSettings.flow_direction_threshold,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=1e-13,
+        max_value=1e-2,
+    ),
+    RangeCheck(
+        error_code=1115,
+        column=models.NumericalSettings.general_numerical_threshold,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=1e-13,
+        max_value=1e-7,
+    ),
+    RangeCheck(
+        error_code=1116,
+        column=models.NumericalSettings.max_nonlin_iterations,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=1,
+    ),
+    RangeCheck(
+        error_code=1117,
+        column=models.NumericalSettings.max_degree,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=1,
+    ),
+    RangeCheck(
+        error_code=1118,
+        column=models.NumericalSettings.minimum_friction_velocity,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=0,
+        max_value=1,
+    ),
+    RangeCheck(
+        error_code=1119,
+        column=models.NumericalSettings.minimum_surface_area,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=1e-13,
+        max_value=1e-7,
+    ),
+    RangeCheck(
+        error_code=1120,
+        column=models.NumericalSettings.preissmann_slot,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=1121,
+        column=models.NumericalSettings.pump_implicit_ratio,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=0,
+        max_value=1,
+    ),
+    RangeCheck(
+        error_code=1122,
+        column=models.NumericalSettings.thin_water_layer_definition,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=1123,
+        column=models.NumericalSettings.use_of_cg,
+        filters=models.NumericalSettings.global_settings != None,
+        min_value=1,
+    ),
+    RangeCheck(
+        error_code=1124,
+        column=models.GlobalSetting.flooding_threshold,
+        min_value=0,
+        max_value=0.3,
+    ),
+    RangeCheck(
+        error_code=1124,
+        level=CheckLevel.WARNING,
+        column=models.GlobalSetting.flooding_threshold,
+        min_value=0,
+        max_value=0.05,
+    ),
+    QueryCheck(
+        error_code=1125,
+        column=models.NumericalSettings.thin_water_layer_definition,
+        invalid=Query(models.NumericalSettings).filter(
+            (models.NumericalSettings.global_settings != None)
+            & (models.NumericalSettings.frict_shallow_water_correction == 3)
+            & (models.NumericalSettings.thin_water_layer_definition <= 0)
+        ),
+        message="v2_numerical_settings.thin_water_layer_definition must be greater than 0 when using frict_shallow_water_correction option 3.",
+    ),
+    QueryCheck(
+        error_code=1126,
+        column=models.NumericalSettings.thin_water_layer_definition,
+        invalid=Query(models.NumericalSettings).filter(
+            (models.NumericalSettings.global_settings != None)
+            & (models.NumericalSettings.limiter_slope_crossectional_area_2d == 3)
+            & (models.NumericalSettings.thin_water_layer_definition <= 0)
+        ),
+        message="v2_numerical_settings.thin_water_layer_definition must be greater than 0 when using limiter_slope_crossectional_area_2d option 3.",
+    ),
+    QueryCheck(
+        error_code=1127,
+        column=models.NumericalSettings.thin_water_layer_definition,
+        invalid=Query(models.NumericalSettings).filter(
+            (models.NumericalSettings.global_settings != None)
+            & (models.NumericalSettings.limiter_slope_friction_2d == 0)
+            & (models.NumericalSettings.limiter_slope_crossectional_area_2d != 0)
+        ),
+        message="v2_numerical_settings.limiter_slope_friction_2d may not be 0 when using limiter_slope_crossectional_area_2d.",
+    ),
+]
+
+
+## 115x SIMULATION SETTINGS, aggregation
+
+CHECKS += [
+    QueryCheck(
+        error_code=1150,
+        column=models.AggregationSettings.aggregation_method,
+        invalid=Query(models.AggregationSettings).filter(
+            (models.AggregationSettings.global_settings_id != None)
+            & (models.AggregationSettings.aggregation_method == "current")
+            & (
+                models.AggregationSettings.flow_variable.notin_(
+                    ("volume", "interception")
+                )
+            )
+        ),
+        message="v2_aggregation_settings.aggregation_method can only be 'current' for 'volume' or 'interception' flow_variables.",
+    )
+]
+
+## 12xx  SIMULATION, timeseries
+CHECKS += [
+    TimeseriesCheck(col, error_code=1200)
     for col in [
         models.BoundaryCondition1D.timeseries,
         models.BoundaryConditions2D.timeseries,
@@ -982,17 +1317,6 @@ CHECKS += [
         models.Lateral2D.timeseries,
     ]
 ]
-CHECKS += [
-    FileExistsCheck(
-        error_code=1109,
-        column=models.GlobalSetting.initial_waterlevel_file,
-    ),
-    FileExistsCheck(
-        error_code=1110,
-        column=models.GlobalSetting.initial_groundwater_level_file,
-        filters=(models.GlobalSetting.groundwater_settings_id != None),
-    ),
-]
 
 
 class Config:
